import { av as LinkDescriptor, as as MetaDescriptor, aJ as ServerDataFrom, aK as ClientDataFrom, aL as Func, aM as Equal, aN as Pretty } from '../../route-data-BZ4q5j1Q.js';
import { A as AppLoadContext } from '../../data-CQbyyGzl.js';
import 'react';

type IsDefined<T> = Equal<T, undefined> extends true ? false : true;
type RouteModule = {
    loader?: Func;
    clientLoader?: Func;
    action?: Func;
    clientAction?: Func;
    HydrateFallback?: unknown;
    default?: unknown;
    ErrorBoundary?: unknown;
};
type LinkDescriptors = LinkDescriptor[];
type RouteInfo = {
    id: unknown;
    params: unknown;
    loaderData: unknown;
};
type MetaMatch<T extends RouteInfo> = Pretty<Pick<T, "id" | "params"> & {
    pathname: string;
    meta: MetaDescriptor[];
    data: T["loaderData"];
    handle?: unknown;
    error?: unknown;
}>;
type MetaMatches<T extends RouteInfo[]> = T extends [infer F extends RouteInfo, ...infer R extends RouteInfo[]] ? [MetaMatch<F>, ...MetaMatches<R>] : [];
type CreateMetaArgs<Params, LoaderData, Parents extends RouteInfo[]> = {
    location: Location;
    params: Params;
    data: LoaderData;
    error?: unknown;
    matches: MetaMatches<Parents>;
};
type MetaDescriptors = MetaDescriptor[];
type IsHydrate<ClientLoader> = ClientLoader extends {
    hydrate: true;
} ? true : ClientLoader extends {
    hydrate: false;
} ? false : false;
type CreateLoaderData<T extends RouteModule> = _CreateLoaderData<ServerDataFrom<T["loader"]>, ClientDataFrom<T["clientLoader"]>, IsHydrate<T["clientLoader"]>, T extends {
    HydrateFallback: Func;
} ? true : false>;
type _CreateLoaderData<ServerLoaderData, ClientLoaderData, ClientLoaderHydrate extends boolean, HasHydrateFallback> = [
    HasHydrateFallback,
    ClientLoaderHydrate
] extends [true, true] ? IsDefined<ClientLoaderData> extends true ? ClientLoaderData : undefined : [
    IsDefined<ClientLoaderData>,
    IsDefined<ServerLoaderData>
] extends [true, true] ? ServerLoaderData | ClientLoaderData : IsDefined<ClientLoaderData> extends true ? ClientLoaderData : IsDefined<ServerLoaderData> extends true ? ServerLoaderData : undefined;
type CreateActionData<T extends RouteModule> = _CreateActionData<ServerDataFrom<T["action"]>, ClientDataFrom<T["clientAction"]>>;
type _CreateActionData<ServerActionData, ClientActionData> = Awaited<[
    IsDefined<ServerActionData>,
    IsDefined<ClientActionData>
] extends [true, true] ? ServerActionData | ClientActionData : IsDefined<ClientActionData> extends true ? ClientActionData : IsDefined<ServerActionData> extends true ? ServerActionData : undefined>;
type ClientDataFunctionArgs<Params> = {
    request: Request;
    params: Params;
};
type ServerDataFunctionArgs<Params> = ClientDataFunctionArgs<Params> & {
    context: AppLoadContext;
};
type CreateServerLoaderArgs<Params> = ServerDataFunctionArgs<Params>;
type CreateClientLoaderArgs<Params, T extends RouteModule> = ClientDataFunctionArgs<Params> & {
    serverLoader: () => Promise<ServerDataFrom<T["loader"]>>;
};
type CreateServerActionArgs<Params> = ServerDataFunctionArgs<Params>;
type CreateClientActionArgs<Params, T extends RouteModule> = ClientDataFunctionArgs<Params> & {
    serverAction: () => Promise<ServerDataFrom<T["action"]>>;
};
type CreateHydrateFallbackProps<Params> = {
    params: Params;
};
type CreateComponentProps<Params, LoaderData, ActionData> = {
    params: Params;
    loaderData: LoaderData;
    actionData?: ActionData;
};
type CreateErrorBoundaryProps<Params, LoaderData, ActionData> = {
    params: Params;
    error: unknown;
    loaderData?: LoaderData;
    actionData?: ActionData;
};

export type { CreateActionData, CreateClientActionArgs, CreateClientLoaderArgs, CreateComponentProps, CreateErrorBoundaryProps, CreateHydrateFallbackProps, CreateLoaderData, CreateMetaArgs, CreateServerActionArgs, CreateServerLoaderArgs, LinkDescriptors, MetaDescriptors };
